using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;
using System.Web.UI.WebControls;
using System.Web.UI;

using NI.Email.Mime;
using NI.Email.Mime.Message;
using NI.Email.Mime.Field;

public partial class ViewMessage : System.Web.UI.Page
{
    private MimeMessage _Message;

    static readonly Regex EmbeddedImageRegex = new Regex(@"<img.*src=""cid:(?<imgCid>.*?)""[^>]*>", RegexOptions.IgnoreCase | RegexOptions.Singleline | RegexOptions.Compiled);
    string EmbeddedImageRegexReplace = @"<img src=""ViewAttachment.aspx?embedded=true&attachcid=${imgCid}&filename=___CURRENT_FILE___""/>";

    static readonly Regex EmbeddedHrefRegex = new Regex(@"<a.*href=""cid:(?<aCid>.*?)""[^>]*>", RegexOptions.Compiled | RegexOptions.Singleline | RegexOptions.IgnoreCase);
    string EmbeddedHrefRegexReplace = @"<a href=""ViewAttachment.aspx?embedded=true&attachcid=${aCid}&filename=___CURRENT_FILE___"">${aCid}</a>";

    protected bool ShowHeader
    {
        get 
        {
            try
            {
                return Convert.ToBoolean(ViewState["showheader"]);
            }
            catch
            {
                return false;
            }
        }
    }

    protected bool ShowBody
    {
        get
        {
            try
            {
                return Convert.ToBoolean(ViewState["showbody"]);
            }
            catch
            {
                return false;
            }
        }
    }

    protected MimeMessage Message
    {
        get { return _Message; }
    }

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);
        string filename = Convert.ToString(Request["filename"]);
		if (filename == null)
		{
			Response.Write("File's not specified or invalid. Please, use following format : <b>ViewMessage.aspx?filename=your_file_name_here</b>. File should be in the web root directory");
			Response.End();
		}
        if (filename.Trim() != String.Empty)
        {
            //System.Diagnostics.Debugger.Break();
            Page.MapPath(Page.AppRelativeVirtualPath);
            filename = Regex.Replace(Page.MapPath(Page.AppRelativeVirtualPath), @"\\[^.]+[.]aspx$", @"\" + filename, RegexOptions.Compiled | RegexOptions.Singleline);
            _Message = new MimeMessage(new FileStream(filename, FileMode.Open, FileAccess.Read));
        }
        if (!IsPostBack)
        {
            DataBind();
        }
    }

    protected void OnViewHeaderClick(object sender, CommandEventArgs c)
    {
        bool currentState = false;
        try
        {
            currentState = Convert.ToBoolean(ViewState["showheader"]);
        }
        catch
        { }
        ViewState.Add("showheader", !currentState);
        DataBind();
    }

    protected void OnViewBodyClick(object sender, CommandEventArgs c)
    {
        bool currentState = false;
        try
        {
            currentState = Convert.ToBoolean(ViewState["showbody"]);
        }
        catch
        { }
        ViewState.Add("showbody", !currentState);
        DataBind();
    }

    protected string ShowMessageBody()
    {
        string result = String.Empty;
        if (Message.IsMimeType("multipart/related"))
        {
            foreach (BodyPart bp in (Message.Body as Multipart).BodyParts)
            {
                if (bp.Body is ITextBody)
                {
                    if (bp.MimeType == ContentTypeField.TypeTextPlain)
                    {
                        result += "<pre>";
                        result += (bp.Body as ITextBody).Reader.ReadToEnd();
                        result += "</pre><br/><hr/><br/>";
                    }
                    else if (bp.MimeType == ContentTypeField.TypeTextHtml)
                    {
                        result += PrepareEmbeddedLinks((bp.Body as ITextBody).Reader.ReadToEnd());
                        result += "<br/><hr/><br/>";
                    }
                }
            }
        }
        else if (Message.Body is Multipart)
        {
            foreach (BodyPart bp in (Message.Body as Multipart).BodyParts)
            {
                if (bp.Body is ITextBody)
                {
                    if (bp.MimeType == ContentTypeField.TypeTextPlain)
                    {
                        result += "<pre>";
                        result += (bp.Body as ITextBody).Reader.ReadToEnd();
                        result += "</pre><br/><hr/><br/>";
                    }
                    else
                    {
                        result += (bp.Body as ITextBody).Reader.ReadToEnd();
                        result += "<br/><hr/><br/>";
                    }
                }
            }
        }
        else if (Message.Body is ITextBody)
        {
            result += (Message.Body as ITextBody).Reader.ReadToEnd();
            result += "<br/><hr/><br/>";
        }
        return result;
    }

    protected string PrepareEmbeddedLinks(string s)
    {
        //System.Diagnostics.Debugger.Break();
        s = EmbeddedImageRegex.Replace(s, EmbeddedImageRegexReplace);
        s = EmbeddedHrefRegex.Replace(s, EmbeddedHrefRegexReplace);
        s = s.Replace("___CURRENT_FILE___", Convert.ToString(Request["filename"]));
        return s;
    }
	
	protected ArrayList GetAttachmentLinks(Entity mimeEntity)
	{
		ArrayList result = GetAttachmentLinksInternal(mimeEntity, 0);
		return result.Count > 0 ? result : null;
	}

    protected ArrayList GetAttachmentLinksInternal(Entity mimeEntity, int indexOffset)
    {
        //System.Diagnostics.Debugger.Break();
        ArrayList result = new ArrayList();
        if (mimeEntity.IsMultipart)
        {
            IList bps = ((Multipart)mimeEntity.Body).BodyParts;
            for (int i = 0; i < bps.Count; ++i)
            {
                BodyPart currentBp = bps[i] as BodyPart;
                ContentTypeField contentType = currentBp.Header.GetField(MimeField.ContentType) as ContentTypeField;
                if (contentType == null) continue;

                if (currentBp.Body is MimeMessage)
                {
                    result.Add(i + indexOffset);
                    result.AddRange(GetAttachmentLinksInternal((MimeMessage)currentBp.Body, i + indexOffset));
                }
                else if (!(currentBp.Body is ITextBody) && !currentBp.Parent.IsMimeType("multipart/related"))
                {
                    result.Add(i + indexOffset);
                }
            }
        }
        return result;
    }
	
	protected bool HasMessageAttachments(Entity message)
	{
		ArrayList result = GetAttachmentLinksInternal(message, 0);
		return result != null && result.Count > 0;
	}
}
